/*
FILE 
	main.c
	$Id: main.c 59 2006-09-11 14:41:43Z ggw $
PURPOSE
	Main cgi interface.
AUTHOR
	GPL License applies, see www.fsf.org for details.
	See LICENSE file included in this distribution.
	(C) 2001-2006 Gary Wallis.
OTHER
	Only Linux and Solaris supported by openisp.net. Please share your 
	ports with us.
HELP
	support @ openisp . net

*/

#include "mysqlrad.h"
#include <ctype.h>
#ifdef Solaris
        char *crypt(char *passwd, char *salt);
#endif

#include "language.h"
#include "local.h"

 //Global vars

#define SHOWPAGE 30
MYSQL mysql;
MYSQL mysql2;
unsigned long rowid;
unsigned start;
unsigned end;
unsigned i;
unsigned n=SHOWPAGE;
char command[100];
char filter[100];
char find[100];
char table[100];
int mode;

int uPermLevel=0;
unsigned uLoginClient=0;
unsigned uReseller=0;
char cUser[100]={""};
char cHost[100]={""};
char gcHostname[100]={""};

#ifdef USECONF

char gcDbIp[16]={""};
char *gcEffectiveDbIp=DBIP;
char gcDbName[100]={DBNAME};
char gcDbLogin[100]={DBLOGIN};
char gcDbPasswd[32]={DBPASSWD};
unsigned guDbPort=DBPORT;
char gcDbSocket[100]={""};
char *gcEffectiveDbSocket=DBSOCKET;

void ReadAndSetConfFileGlobalVars(void);
#endif


 //SSLLoginCookie()
char cCookie[1024]={""};
char cLogin[100]={""};
char cPasswd[100]={""};
unsigned uSSLCookieLogin=0;

char function[100]={""};
char query[8192]={""};
char *qstr=query;

char *gcBuildInfo="$Id: main.c 59 2006-09-11 14:41:43Z ggw $";
char *gcRADStatus="Released 1.94";

 //Local
void Footer_ism3(void);
void Header_ism3(char *msg, int js);
const char *ForeignKey(const char *cTableName, const char *cFieldName, unsigned uKey);
char *EmailInput(char *cInput);
void GetClientOwner(unsigned uClient, unsigned *uOwner);

 //Ext
int ExtMainCommands(pentry entries[], int x);
void ExtMainContent(void);
void ExtMainShell(int argc, char *argv[]);
void ExttZoneNavBar(void);
void ExttResourceNavBar(void);
void ExttJobNavBar(void);
void ExttNameServerNavBar(void);
void ExttConfigurationNavBar(void);
void ExttClientNavBar(void);

 //Only local
void ConnectDb(void);
void NoSuchFunction(void);
void mysqlBind(const char *result);
void InitialInstall(void);
char *UserLevel(unsigned uPermLevel);

int ValidLogin(int mode);
void SSLCookieLogin(void);
void SetLogin(void);
void EncryptPasswdWithSalt(char *pw, char *salt);
void GetPLAndClient(char *cUser);
void htmlSSLLogin(void);


int main(int argc, char *argv[])
{
	pentry entries[64];
	entry gentries[16];
	char *gcl;
	register int x;
	int cl=0;

#if defined(Linux)
	gethostname(gcHostname, 98);
#else
	//Solaris
	sysinfo(SI_HOSTNAME, gcHostname, 98);
#endif

#ifdef USECONF
	ReadAndSetConfFileGlobalVars();
#endif

	if(getenv("REMOTE_HOST")!=NULL)
	{
		sprintf(cHost,"%.99s",getenv("REMOTE_HOST"));
	}
	else if(getenv("REMOTE_ADDR")!=NULL)
	{
		sprintf(cHost,"%.99s",getenv("REMOTE_ADDR"));
	}
	else
	{
		ExtMainShell(argc,argv);//Must exit()
	}

	ConnectDb();

	//Get
	if(strcmp(getenv("REQUEST_METHOD"),"POST"))
	{

		SSLCookieLogin();

	        gcl = getenv("QUERY_STRING");

	        for(x=0;gcl[0] != '\0' && x<8;x++)
		{
               		getword(gentries[x].val,gcl,'&');
               		plustospace(gentries[x].val);
               		unescape_url(gentries[x].val);
               		getword(gentries[x].name,gentries[x].val,'=');

			//basic anti hacker
			escape_shell_cmd(gentries[x].val);

			//Local vars
			if(!strcmp(gentries[x].name,"function")) 
				sprintf(function,"%.99s",gentries[x].val);
		}

		if(function[0])
		{
			if(!strncmp(function,"Main",4))
#ifdef USECONF
			{
				sprintf(query,"%s using %s",RELEASE,USECONF);
				mysqlBind(query);
			}
#else
			mysqlBind(RELEASE);
#endif
			else if(!strncmp(function,"Logout",6))
			{
				printf("Set-Cookie: mysqlBindLogin=; expires=0\n");
				printf("Set-Cookie: mysqlBindPasswd=; expires=0\n");
				cCookie[0]=0;
                                uPermLevel=0;
                                uLoginClient=0;
                                cUser[0]=0;
                                uSSLCookieLogin=0;
                                htmlSSLLogin();
			}
			else if(!strncmp(function,"tZone",5))
				ExttZoneGetHook(gentries,x);
			else if(!strncmp(function,"tResource",9))
				ExttResourceGetHook(gentries,x);
			else if(!strncmp(function,"tRRType",7))
				ExttRRTypeGetHook(gentries,x);
			else if(!strncmp(function,"tJob",4))
				ExttJobGetHook(gentries,x);
			else if(!strncmp(function,"tNameServer",11))
				ExttNameServerGetHook(gentries,x);
			else if(!strncmp(function,"tMailServer",11))
				ExttMailServerGetHook(gentries,x);
			else if(!strncmp(function,"tConfiguration",14))
				ExttConfigurationGetHook(gentries,x);
			else if(!strncmp(function,"tClient",7))
				ExttClientGetHook(gentries,x);
			else if(!strncmp(function,"tAuthorize",10))
				ExttAuthorizeGetHook(gentries,x);
		}

		mysqlBind(ISPNAME);

	}//end get method interface section

	//Post method interface
	cl = atoi(getenv("CONTENT_LENGTH"));
	for(x=0;cl && (!feof(stdin)) && x<64 ;x++)
	{
		entries[x].val = fmakeword(stdin,'&',&cl);
		plustospace(entries[x].val);
		unescape_url(entries[x].val);
		entries[x].name = makeword(entries[x].val,'=');

		//basic anti hacker
		//Allow posted page cursors. See PageMachine()
		//escape_shell_cmd(entries[x].val);

		if(!strcmp(entries[x].name,"function")) 
			sprintf(function,"%.99s",entries[x].val);
		else if(!strcmp(entries[x].name,"cLogin"))
			sprintf(cLogin,"%.99s",entries[x].val);
                else if(!strcmp(entries[x].name,"cPasswd"))
			sprintf(cPasswd,"%.99s",entries[x].val);
	}

	//SSLCookieLogin()
        if(!strcmp(function,"Login")) SetLogin();

        if(!uPermLevel || !cUser[0] || !uLoginClient)
                SSLCookieLogin();

	//Main Post Menu
	tZoneCommands(entries,x);
	tResourceCommands(entries,x);
	tRRTypeCommands(entries,x);
	tJobCommands(entries,x);
	tNameServerCommands(entries,x);
	tMailServerCommands(entries,x);
	tConfigurationCommands(entries,x);
	tClientCommands(entries,x);
	tAuthorizeCommands(entries,x);


	ExtMainCommands(entries,x);

	NoSuchFunction();

	return(0);

}//end of main()

#include "mainfunc.h"

void mysqlBind(const char *result)
{
	Header_ism3("Main",0);

	if(result[0])
		printf("%s\n",result);
	else
		ExtMainContent();	

	Footer_ism3();

}//void mysqlBind(char *result)


void StyleSheet(void)
{
        printf("<style type=\"text/css\">\n");
        printf("<!--\n");

        printf("input.smallButton {width:50px;}\n");
        printf("input.medButton {width:100px;}\n");
        printf("input.largeButton {width:150px;}\n");
        printf("input.warnButton {color:red;}\n");
        printf("input.lwarnButton {color:red;width:150px;}\n");
        printf("input.alertButton {color:yellow;}\n");
        printf("input.lalertButton {color:yellow;width:150px;}\n");
        printf("input.revButton {color:white;background:black;}\n");
        printf("input.lrevButton {color:white;background:black;width:150px;}\n");
        printf("A.darkLink {color:black; text-decoration:none;}\n");
	printf("A.darkLink:hover {color:blue; text-decoration:underline;}\n");
        printf("A:hover {color:blue; text-decoration:underline;}\n");

	printf("textarea {font-family: Arial,Helvetica; font-size: 11px;}\n");
	printf("pre {font-family: Arial,Helvetica; font-size: 11px;}\n");
	printf("input {font-family: Arial,Helvetica; font-size: 11px;}\n");
	printf("select {font-family: Arial,Helvetica;}\n");

	//printf("body {font-family: Arial,Helvetica; font-size: 11px;}\n");

	printf("body\n");
	printf("{\n");
	printf("	margin-top: 0px;\n");
	printf("	margin-bottom: 12px;\n");
	printf("	margin-left: 12px;\n");
	printf("	margin-right: 10px;\n");
	printf("	font-family: Arial,Helvetica;\n");
	printf("	#font-size: 85%%;\n");
	printf("	font-size: 65%%;\n");
	printf("	line-height: 135%%;\n");
	printf("	padding: 0px;\n");
	printf("}\n");

	printf("br.clearall { clear: both; }\n");

	printf("td { font-family: Arial,Helvetica; }\n");
	
	printf("#menuholder\n");
	printf("{\n");
	printf("	#width: 100%%;\n");
	printf("}\n");
	
	printf("#menutab\n");
	printf("{\n");
	printf("	background: #e5e5e5 url('http://unixservice.com/images/hairline.gif') repeat-x bottom center;\n");
	printf("	border-top: 0px solid #BEBFBF;\n");
	printf("	border-right: 1px solid #BEBFBF;\n");
	printf("	margin-top: 8px;\n");
	printf("}\n");
	
	printf("#topline\n");
	printf("{\n");
	printf("	#width: 100%%;\n");
	printf("	background: transparent url('http://unixservice.com/images/hairline.gif') repeat-x top center;\n");
	printf("}\n");
	
	printf("#menutab ul, #menutab ol\n");
	printf("{\n");
	printf("	margin: 0;\n");
	printf("	padding: 0px 0px 0;\n");
	printf("	list-style: none;\n");
	printf("}\n");
	
	printf("#menutab li\n");
	printf("{\n");
	printf("	float: left;\n");
	printf("	background: url('http://unixservice.com/images/left.gif') no-repeat left top;\n");
	printf("	margin: 0;\n");
	printf("	padding: 0 0 0 9px;\n");
	printf("}\n");
	
	printf("#menutab a, #menutab a.last\n");
	printf("{\n");
	printf("	float:left;\n");
	printf("	display: block;\n");
	printf("	background: url('http://unixservice.com/images/right.gif') no-repeat right top;\n");
	printf("	padding: 5px 15px 2px 6px;\n");
	printf("	color: #222;\n");
	printf("	text-decoration: none;\n");
	printf("}\n");
	
	printf(" /* Commented Backslash Hack\n");
	printf("	hides rule from IE5-Mac \\*/\n");
	printf("	#menutab a, #menutab a.last {float:none;}\n");
	printf("/* End IE5-Mac hack */\n");
	
	printf("#menutab a.last\n");
	printf("{\n");
	printf("	background: url('http://unixservice.com/images/right_last.gif') no-repeat right top;\n");
	printf("	padding: 5px 17px 2px 6px;\n");
	printf("}\n");
	
	printf("#menutab a:hover\n");
	printf("{\n");
	printf("	color: black;\n");
	printf("	text-decoration: underline;\n");
	printf("}\n");
	
	printf("#menutab #current\n");
	printf("{\n");
	printf("	background-image: url('http://unixservice.com/images/left_on.gif');\n");
	printf("}\n");
	
	printf("#menutab #current a\n");
	printf("{\n");
	printf("	background-image: url('http://unixservice.com/images/right_on.gif');\n");
	printf("	padding-bottom: 3px;\n");
	//printf("	color: #900;\n");
	printf("	text-decoration: none;\n");
	//printf("	font-weight: bold;\n");
	printf("}\n");

        printf("-->\n");
        printf("</style>\n");

}//void StyleSheet(void)

char *UserLevel(unsigned uPermLevel)
{
	switch(uPermLevel)
	{
		case 12:
			return("Root");
		break;
		case 10:
			return("Admin");
		break;
		case 8:
			return("Reseller");
		break;
		case 7:
			return("Customer");
		break;
		default:
			return("---");
		break;
	}

}//char *UserLevel(unsigned uPermLevel)


void Header_ism3(char *title, int js)
{
	printf("Content-type: text/html\n\n");
	printf("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n");
        printf("<html><head><title>mysqlBind %s </title>",title);
	printf("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">\n");
	StyleSheet();
	printf("</head><body><form action=mysqlBind.cgi method=post><blockquote>\n");

	//Table nav bar
	if(!strncmp(function,"tZone",5))
		ExttZoneNavBar();
	else if(!strncmp(function,"tResource",7))
		ExttResourceNavBar();
	else if(!strncmp(function,"tJob",4))
		ExttJobNavBar();
	else if(!strncmp(function,"tNameServer",7))
		ExttNameServerNavBar();
	else if(!strncmp(function,"tConfiguration",7))
		ExttConfigurationNavBar();
	else if(!strncmp(function,"tClient",7))
		ExttClientNavBar();
	else if(!strncmp(function,"tMailServer",7))
		ExttMailServerNavBar();
	else if(!strncmp(function,"tAuthorize",7))
		ExttAuthorizeNavBar();
	else if(!strncmp(function,"tRRType",7))
		ExttRRTypeNavBar();
	
	//Login info
	printf("<font size=3><b>mysqlBind</b></font> \n");
	printf("&nbsp;&nbsp;&nbsp;<font color=red>%s logged in from %s [%s]</font>",
							cUser,cHost,UserLevel(uPermLevel));

	//Logout link
	if(uSSLCookieLogin)
		printf(" <a title='Erase login cookies' href=mysqlBind.cgi?function=Logout>Logout</a> ");

	//Generate Menu Items
	printf("\n<!-- tab menu -->\n");
	printf("<div id=menuholder>\n");
	printf("\t<div id=menutab>\n");

	printf("\t\t<div id=topline>\n");
	printf("\t\t\t<ol>\n");

	//Main
	printf("\t\t\t<li");
	if(strncmp(function,"Main",4))
		printf(">\n");
	else
		printf(" id=current>\n");
	printf("\t\t\t<a title='Home start page' href=mysqlBind.cgi?function=Main>Main</a>\n");
	printf("\t\t\t</li>\n");
	
	//tZone
	if(uPermLevel>=7)
	{
	printf("\t\t\t<li");
	if(strncmp(function,"tZone",5) && strncmp(find,"tResource",7))
		printf(">\n");
	else
		printf(" id=current>\n");
	printf("\t\t\t<a title='DNS Zones' href=mysqlBind.cgi?function=tZone>tZone</a>\n");
	printf("\t\t\t</li>\n");
	}
		
	//tResource
	if(uPermLevel>=7)
	{
	printf("\t\t\t<li");
	if(!strncmp(find,"tResource",7) || !strncmp(function,"tResource",7))
		printf(" id=current>\n");
	else
		printf(">\n");
	printf("<a title='Resource Records for a given Zone' href=mysqlBind.cgi?function=tResource>tResource</a>\n");
	printf("\t\t\t</li>\n");
	}
		
	//tJob
	if(uPermLevel>=7)
	{
	printf("\t\t\t<li");
	if(strncmp(function,"tJob",4))
		printf(">\n");
	else
		printf(" id=current>\n");
	printf("\t\t\t<a title='Job Queue' href=mysqlBind.cgi?function=tJob>tJob</a>\n");
	printf("\t\t\t</li>\n");
	}
		
	//tNameServer
	if(uPermLevel>=10)
	{
	printf("\t\t\t<li");
	if(strncmp(function,"tNameServer",7))
		printf(">\n");
	else
		printf(" id=current>\n");
	printf("\t\t\t<a title='Name server groups' href=mysqlBind.cgi?function=tNameServer>tNameServer</a>\n");
	printf("\t\t\t</li>\n");
	}

	//tMailServer
	if(uPermLevel>=10)
	{
	printf("\t\t\t<li");
	if(strncmp(function,"tMailServer",7))
		printf(">\n");
	else
		printf(" id=current>\n");
	printf("\t\t\t<a title='Name server groups' href=mysqlBind.cgi?function=tMailServer>tMailServer</a>\n");
	printf("\t\t\t</li>\n");
	}


	//tConfiguration
	if(uPermLevel>=10)
	{
	printf("\t\t\t<li");
	if(strncmp(function,"tConfiguration",7))
		printf(">\n");
	else
		printf(" id=current>\n");
	printf("\t\t\t<a title='Runtime configuration variables' href=mysqlBind.cgi?function=tConfiguration>tConfiguration</a>\n");
	printf("\t\t\t</li>\n");
	}

	//tClient
	if(uPermLevel>=10)
	{
	printf("\t\t\t<li");
	if(strncmp(function,"tClient",7))
		printf(">\n");
	else
		printf(" id=current>\n");
	printf("<a class=last title='Clients' href=mysqlBind.cgi?function=tClient>tClient</a>\n");
	printf("\t\t\t</li>\n");
	}
	
	printf("\t\t\t</ol>\n");

	printf("\t\t\t<br class=clearall >\n");
	printf("\t\t</div>\n");
	printf("\t</div>\n");
	printf("</div>\n");

}//Header_ism3(char *title, int js)


void Footer_ism3(void)
{
	printf("</form></blockquote>");

	exit(0);

}//Footer_ism3(void)


//MYSQL *mysql_real_connect(MYSQL *mysql, const char *host, const char *user, const char *passwd, const char *db, unsigned int port, const char *unix_socket, unsigned long client_flag)
void ConnectDb(void)
{

        mysql_init(&mysql);
#ifdef USECONF
        if(!mysql_real_connect(&mysql,gcEffectiveDbIp,gcDbLogin,gcDbPasswd,
					gcDbName,guDbPort,gcEffectiveDbSocket,0))
#else
        if(!mysql_real_connect(&mysql,DBIP,DBLOGIN,DBPASSWD,DBNAME,DBPORT,DBSOCKET,0))
#endif
        {
		//Hummm this maybe an easier solution than TextConnectDb()
		if(cHost[0])
                	mysqlBind("Database server unavailable. Did you run mysqlBind.cgi Initialize &lt;mysqlpwd&gt;?");
		else
			fprintf(stderr,"Database server unavailable. DBLOGIN:%s: DBPASSWD:%s: DBNAME:%s:",DBLOGIN,DBPASSWD,DBNAME);
        }

}//end of ConnectDb()


void NoSuchFunction(void)
{
	 
	sprintf(query,"[%s] Not Recognized",function);
	mysqlBind(query);
}

void ProcessControlVars(pentry entries[], int x)
{
	register int i;

	for(i=0;i<x;i++)
	{
		if(!strcmp(entries[i].name,"filter"))
			strcpy(filter,entries[i].val);
		else if(!strcmp(entries[i].name,"command"))
			strcpy(command,entries[i].val);
		else if(!strcmp(entries[i].name,"find"))
			strcpy(find,entries[i].val);
		else if(!strcmp(entries[i].name,"rowid"))
			sscanf(entries[i].val,"%lu",&rowid);
	}
}

void PageMachine(char *funcName, int lmode, char *msg)
{
	
if(lmode)
{
	//List mode
	
	if(rowid<1) rowid=1;
	if(rowid>(n=((i/SHOWPAGE)+1))) rowid=n;

	if(!strcmp(find," >"))
	{
		//If NOT on last page show next page
		if( rowid >= n-1 )
		{
			//If on last page adjust end
			start= ((i/SHOWPAGE)*SHOWPAGE) + 1;
			end=i;
		}
		else
		{
			start=((rowid)*SHOWPAGE)+1;
			end=start+SHOWPAGE-1;
			rowid++;
		}
	}
	else if(!strcmp(find,"< "))
	{
		if(rowid>1 )
		{
			rowid--;
			start=(rowid)*SHOWPAGE-SHOWPAGE+1;
			end=start+SHOWPAGE-1;
		}
		else
		{
			start=1;
			if(i > SHOWPAGE)
			{
				end=SHOWPAGE;
			}
			else
			{
				end=i;
			}
			rowid=1;
		}
	}
	else if(!strcmp(find,">>"))
	{
		start= ((i/SHOWPAGE)*SHOWPAGE) + 1;
		end=i;
		rowid=n;
	}
	else if(1)
	{
		start=1;
		if(i > SHOWPAGE)
		{
			end=SHOWPAGE;
		}
		else
		{
			end=i;
		}
		rowid=1;
	}

	Header_ism3(funcName,0);

	if(!i) 
        {
                printf(LANG_PAGEMACHINE_HINT);

        }
 

        //printf("</center><form action=mysqlBind.cgi method=post >\n");

        //printf("<font size=2><table width=100%% border=0 ><tr><td>");
        //printf("<input style='width:30px;' type=submit name=find value=\"<<\" >");
        //printf("<input style='width:30px;' type=submit name=find value=\"< \" >");
        //printf(LANG_PAGEMACHINE_SEARCH);
        //printf("<input style='width:30px;' type=submit name=find value=\" >\" >");
        //printf("<input style='width:30px;' type=submit name=find value=\">>\" >\n");
        printf(LANG_PAGEMACHINE_SHOWING,1+(start/SHOWPAGE),n,start,end,i);


	printf("<input type=hidden name=rowid value=%lu>",rowid);
	printf("<input type=hidden name=function value=%s >\n",funcName);
	if(table[0])
	{
		printf("<input type=hidden name=table value=%s >\n",table);
		printf(" <font color=blue>%s</font>",table);
	}
	
}
else
{
	//Page mode
	//on entry i has number of rows
	//on entry rowid has current position unless i=1
	//if i=1 then we need to figure out real i

	n=i;

	if(rowid<1)
	{
		rowid=1;
		return;
	}
	
	if(!strcmp(find," >"))
	{
		//If on last page stay there
		if( rowid >= i )
		{
			//If on last page adjust end
			rowid=i;
		}
		else
		{
			rowid++;
		}
	}
	else if(!strcmp(find,"< "))
	{
		if(rowid>1 )
		{
			rowid--;
		}
		else
		{
			rowid=1;
		}
	}
	else if(!strcmp(find,">>"))
	{
		rowid=i;
	}
	else if(!strcmp(find,"<<"))
	{
		rowid=1;
	}
	else if(1)
	{
		//If on last page stay there
		if( rowid >= i )
		{
			rowid=i;
		}
	}

}//end lmode

}//PageMachine()


void OpenFieldSet(char *cLabel, unsigned uWidth)
{
	printf("<fieldset><legend>%s</legend><table width=%u%%>\n",cLabel,uWidth);

}//void OpenFieldSet()


void CloseFieldSet(void)
{
	printf("</table></fieldset>\n");

}//void CloseFieldSet(void)


void OpenRow(char *cFieldLabel, char *cColor)
{
	printf("<tr><td width=20%% valign=top><font color=%.32s>%.32s</td><td>",cColor,cFieldLabel);
}//void OpenRow(char *label)


void tTablePullDownOwner(char *cTableName, char *cFieldName,
                        char *cOrderby, int unsigned uSelector)
{
        char label[128];
        MYSQL_RES *res;         
        MYSQL_ROW field;

        char cSelectName[34]={""};
        char cLocalTableName[256]={""};
        char *cp;

        register int i,n;
      
        if(!cTableName[0] || !cFieldName[0] || !cOrderby[0])
        {
                printf("Invalid input tTablePullDown()");
                return;
        }

        //Extended functionality
        strncpy(cLocalTableName,cTableName,255);
        if((cp=strchr(cLocalTableName,';')))
        {
                strncpy(cSelectName,cp+1,32);
                cSelectName[32]=0;
                *cp=0;
        }


        if(uPermLevel<10)
                sprintf(query,"SELECT _rowid,%s FROM %s WHERE uOwner=%u ORDER BY %s",
                                cFieldName,cLocalTableName,uLoginClient,cOrderby);
        else
                sprintf(query,"SELECT _rowid,%s FROM %s ORDER BY %s",
                                cFieldName,cLocalTableName,cOrderby);


        mysql_query(&mysql,query);
	if(mysql_errno(&mysql))
        {
                printf("%s",mysql_error(&mysql));
                return;
        }
	res=mysql_store_result(&mysql);
	i=mysql_num_rows(res);

	if(cSelectName[0])
                sprintf(label,"%s",cSelectName);
        else
                sprintf(label,"%s_%sPullDown",cLocalTableName,cFieldName);

        if(i>0)
        {
                printf("<select name=%s>\n",label);

                //Default no selection
                printf("<option>---</option>\n");

                for(n=0;n<i;n++)
                {
                        int unsigned field0=0;

                        field=mysql_fetch_row(res);
                        sscanf(field[0],"%u",&field0);

                        if(uSelector != field0)
                        {
                                printf("<option>%s</option>\n",field[1]);
                        }
                        else
                        {
                                printf("<option selected>%s</option>\n",field[1]);
                        }
                }
        }
        else
        {
        printf("<select name=%s><option>---</option></select>\n"
                        ,label);
        }
        printf("</select>\n");

}//tTablePullDownOwner()


void tTablePullDown(char *cTableName, char *cFieldName,
                        char *cOrderby, int unsigned uSelector)
{
        char label[128];
        MYSQL_RES *res;         
        MYSQL_ROW field;

        char cSelectName[34]={""};
        char cLocalTableName[256]={""};
        char *cp;

        register int i,n;
      
        if(!cTableName[0] || !cFieldName[0] || !cOrderby[0])
        {
                printf("Invalid input tTablePullDown()");
                return;
        }

        //Extended functionality
        strncpy(cLocalTableName,cTableName,255);
        if((cp=strchr(cLocalTableName,';')))
        {
                strncpy(cSelectName,cp+1,32);
                cSelectName[32]=0;
                *cp=0;
        }


        sprintf(query,"select _rowid,%s from %s order by %s",
                                cFieldName,cLocalTableName,cOrderby);

        mysql_query(&mysql,query);
	if(mysql_errno(&mysql))
        {
                printf("%s",mysql_error(&mysql));
                return;
        }
	res=mysql_store_result(&mysql);
	i=mysql_num_rows(res);

	if(cSelectName[0])
                sprintf(label,"%s",cSelectName);
        else
                sprintf(label,"%s_%sPullDown",cLocalTableName,cFieldName);

        if(i>0)
        {
                printf("<select name=%s>\n",label);

                //Default no selection
                printf("<option>---</option>\n");

                for(n=0;n<i;n++)
                {
                        int unsigned field0=0;

                        field=mysql_fetch_row(res);
                        sscanf(field[0],"%u",&field0);

                        if(uSelector != field0)
                        {
                                printf("<option>%s</option>\n",field[1]);
                        }
                        else
                        {
                                printf("<option selected>%s</option>\n",field[1]);
                        }
                }
        }
        else
        {
        printf("<select name=%s><option>---</option></select>\n"
                        ,label);
        }
        printf("</select>\n");

}//tTablePullDown()


void tTablePullDownReadOnly(char *cTableName, char *cFieldName,
                        char *cOrderby, int unsigned uSelector)
{
        MYSQL_RES *res;         
        MYSQL_ROW field;

        char cSelectName[34]={""};
        char label[100]={""};
        char cLocalTableName[256]={""};
        char *cp;

        if(!cTableName[0] || !cFieldName[0] || !cOrderby[0])
        {
                printf("Invalid input tTablePullDown()");
                return;
        }

        //Extended functionality
        strncpy(cLocalTableName,cTableName,255);
        if((cp=strchr(cLocalTableName,';')))
        {
                strncpy(cSelectName,cp+1,32);
                cSelectName[32]=0;
                *cp=0;
        }

	sprintf(query,"select _rowid,%s from %s where _rowid=%u",
                                cFieldName,cLocalTableName,uSelector);

        mysql_query(&mysql,query);
        if(mysql_errno(&mysql))
        {
                printf("%s",mysql_error(&mysql));
                return;
        }
        res=mysql_store_result(&mysql);

        if(cSelectName[0])
                sprintf(label,"%s",cSelectName);
        else
                sprintf(label,"%s_%sPullDown",cLocalTableName,cFieldName);


        if(mysql_num_rows(res))
        {
                field=mysql_fetch_row(res);
		printf("<select name=%s disabled>\n",label);		
                printf("<option>%s</option>\n",field[1]);
                printf("<input type=hidden value=\"%s\" name=%s>",
                                                field[1],label);
		printf("</select>");
        }
        else
        {
		printf("<select name=%s disabled>\n",label);
                printf("<option>---</option>\n");
		printf("</select>");
        }

}//tTablePullDownReadOnly()


int ReadPullDown(char *cTableName,char *cFieldName,char *cLabel)
{
        MYSQL_RES *res;
        MYSQL_ROW field;

        unsigned int rowid=0;//Not found

        sprintf(query,"select _rowid from %s where %s='%s'",
                        cTableName,cFieldName,cLabel);
        mysql_query(&mysql,query);
        if(mysql_errno(&mysql)) mysqlBind(mysql_error(&mysql));
        res=mysql_store_result(&mysql);
        if((field=mysql_fetch_row(res)))
        	sscanf(field[0],"%u",&rowid);
        mysql_free_result(res);
        return(rowid);

}//ReadPullDown(char *cTableName,char *cLabel)


const char *TextAreaSave(const char *cField)
{
	register unsigned int i=0,j=0,uNum=0,uCtrlM=0;
	char *cCopy=NULL;

	for(i=0;cField[i];i++)
	{
		if(cField[i]=='\'' || cField[i]=='\\') uNum++;
		if(cField[i]=='\r') uCtrlM++;
	}
	if(!uNum && !uCtrlM) return(cField);

	if(uNum)
		cCopy=(char *)malloc( ( (strlen(cField)) + (uNum*2) + 1 ));
	else
		cCopy=(char *)cField;

	if(!cCopy) mysqlBind("TextAreaInput() malloc error");

	i=0;
	while(cField[i])
	{
		if( cField[i]=='\'' )
		{
			cCopy[j++]='\\';
			cCopy[j++]='\'';
		}
		else if( cField[i]=='\\')
		{
			cCopy[j++]='\\';
			cCopy[j++]='\\';
		}
		
		//Remove nasty ctrl-m's. fsck /u Bill!
		else if(cField[i]!='\r')
		{
			cCopy[j++]=cField[i];
		}
		i++;
	}

	cCopy[j]=0;
	return(cCopy);

}//const char *TextAreaSave(const char *cField)


const char *TransformAngleBrackets(const char *cField)
{
	register unsigned int i=0,j=0,uNum=0;
	char *cCopy=NULL;

	for(i=0;cField[i];i++)
		if(cField[i]=='<' || cField[i]=='>') uNum++;
	if(!uNum) return(cField);
	cCopy=(char *)malloc( ( (strlen(cField)) + (uNum*4) + 1 ));

	if(!cCopy) mysqlBind("TransformAngleBrackets() malloc error");

	i=0;
	while(cField[i])
	{
		//Expand angle brackets into HTML codes
		if( cField[i]=='<' )
		{
			cCopy[j++]='&';
			cCopy[j++]='l';
			cCopy[j++]='t';
			cCopy[j++]=';';
		}
		else if( cField[i]=='>' )
		{
			cCopy[j++]='&';
			cCopy[j++]='g';
			cCopy[j++]='t';
			cCopy[j++]=';';
		}
		else if(1)
                {
                        cCopy[j++]=cField[i];
                }
		i++;
	}

	cCopy[j]=0;
	return(cCopy);

}//const char *TransformAngleBrackets(const char *cField)


const char *EncodeDoubleQuotes(const char *cField)
{
	register unsigned int i=0,j=0,uNum=0;
	char *cCopy=NULL;

	for(i=0;cField[i];i++)
		if(cField[i]=='"') uNum++;
	if(!uNum) return(cField);
	cCopy=(char *)malloc( ( (strlen(cField)) + (uNum*5) + 1 ));

	if(!cCopy) mysqlBind("EncodeDoubleQuotes() malloc error");

	i=0;
	while(cField[i])
	{
		//Expand double quote into HTML codes
		if( cField[i]=='"' )
		{
			cCopy[j++]='&';
			cCopy[j++]='q';
			cCopy[j++]='u';
			cCopy[j++]='o';
			cCopy[j++]='t';
			cCopy[j++]=';';
		}
		else if(1)
                {
                        cCopy[j++]=cField[i];
                }
		i++;
	}

	cCopy[j]=0;
	return(cCopy);

}//const char *EncodeDoubleQuotes(const char *cField)


void YesNo(unsigned uSelect)
{
        if(uSelect)
                printf("Yes");
        else
                printf("No");

}//YesNo()


void YesNoPullDown(char *cFieldName, unsigned uSelect)
{
	printf("<select name=cYesNo%s>\n",cFieldName);
        if(uSelect==0)
                printf("<option selected>No</option>\n");
        else
                printf("<option>No</option>\n");
        if(uSelect==1)
                printf("<option selected>Yes</option>\n");
        else    
                printf("<option>Yes</option>\n");

        printf("</select>\n");

}//YesNoPullDown()


int ReadYesNoPullDown(char *cLabel)
{
        if(!strcmp(cLabel,"Yes"))
                return(1);
        else
                return(0);

}//ReadYesNoPullDown(char *cLabel)


const char *ForeignKey(const char *cTableName, const char *cFieldName, unsigned uKey)
{
        MYSQL_RES *res;
        MYSQL_ROW field;

	static char query[256];

        sprintf(query,"SELECT %s FROM %s WHERE _rowid=%u",
                        cFieldName,cTableName,uKey);
        mysql_query(&mysql,query);
        if(mysql_errno(&mysql)) return(mysql_error(&mysql));

        res=mysql_store_result(&mysql);
        if(mysql_num_rows(res)==1)
        {
                field=mysql_fetch_row(res);
                return(field[0]);
        }

        if(!uKey)
	{
		return("---");
	}
	else
	{
		sprintf(query,"%u",uKey);
		return(query);
	}

}//const char *ForeignKey(const char *cTableName, const char *cFieldName, unsigned uKey)


void InitialInstall(void)
{
	mysqlBind("Please run\
		mysqlBind.cgi Initialize &lt;mysqlpwd&gt; from the command line");

}//void InitialInstall(void)


char *IPNumber(char *cInput)
{
	unsigned a=0,b=0,c=0,d=0;

	sscanf(cInput,"%u.%u.%u.%u",&a,&b,&c,&d);

	if(a>255) a=0;
	if(b>255) b=0;
	if(c>255) c=0;
	if(d>255) d=0;

	sprintf(cInput,"%u.%u.%u.%u",a,b,c,d);

	return(cInput);

}//char *IPNumber(char *cInput)

 //tValidFunc functions: Form validation feedback

char *IsZero(unsigned uInput)
{
        if(uInput)
                return("white");
        else
                return("red");

}//char *IsZero(unsigned uInput)
 
char *BadIPNum(char *cInput)
{ 
        if( cInput!=NULL && cInput[0] && strcmp(cInput,"0.0.0.0"))
                return("white");
        else
                return("red");

}//char *BadIPNum(char *cInput)

char *EmptyString(char *cInput)
{
        if(cInput!=NULL && cInput[0])
                return("white");
        else
                return("red");

}//char *EmptyString(char *cInput) 


 //tInputFunc functions: Save data to table

char *EmailInput(char *cInput)
{
	register int i;

	for(i=0;cInput[i];i++)
	{
	
		if(!isalnum(cInput[i]) && cInput[i]!='.'  && cInput[i]!='-' 
				&& cInput[i]!='@' && cInput[i]!='_')
			break;
		if(isupper(cInput[i])) cInput[i]=tolower(cInput[i]);
	}
	cInput[i]=0;

	return(cInput);

}//char *EmailInput(char *cInput)


char *FQDomainName(char *cInput)
{
	register int i;

	for(i=0;cInput[i];i++)
	{
	
		if(!isalnum(cInput[i]) && cInput[i]!='.'  && cInput[i]!='-' )
			break;
		if(isupper(cInput[i])) cInput[i]=tolower(cInput[i]);
	}
	cInput[i]=0;

	return(cInput);

}//char *FQDomainName(char *cInput)


char *WordToLower(char *cInput)
{
	register int i;

	for(i=0;cInput[i];i++)
	{
	
		if(!isalnum(cInput[i]))	break;
		if(isupper(cInput[i])) cInput[i]=tolower(cInput[i]);
	}
	cInput[i]=0;

	return(cInput);

}//char *WordToLower(char *cInput)

void SetLogin(void)
{
	if( ValidLogin(0) )
	{
		printf("Set-Cookie: mysqlBindLogin=%s;\n",cLogin);
		printf("Set-Cookie: mysqlBindPasswd=%s;\n",cPasswd);
		strncpy(cUser,cLogin,41);
		GetPLAndClient(cUser);
		uSSLCookieLogin=1;
		mysqlBind("Logged in");
	}
	else
	{
		uSSLCookieLogin=0;
		SSLCookieLogin();
	}
				
}//void SetLogin(void)


char *cGetPasswd(char *cLogin);
int ValidLogin(int mode)
{
	char salt[3]={'/','D',0};
	char cPassword[100]={""};

	//Notes:
	//Mode=1 means we have encrypted passwd from cookie

	strcpy(cPassword,cGetPasswd(cLogin));
	if(cPassword[0])
	{
		if(!mode)
		{
			strncpy(salt,cPassword,2);
			EncryptPasswdWithSalt(cPasswd,salt);
			if(!strcmp(cPasswd,cPassword)) return 1;
		}
		else
		{
			if(!strcmp(cPasswd,cPassword)) return 1;
		}
	}
	return 0;

}//ValidLogin()


char *cGetPasswd(char *cLogin)
{
	static char cPasswd[100]={""};
        MYSQL_RES *res;
        MYSQL_ROW field;

	
	sprintf(query,"SELECT cPasswd FROM tAuthorize WHERE cLabel='%s'",
			cLogin);
	mysql_query(&mysql,query);
	if(mysql_errno(&mysql))
		mysqlBind(mysql_error(&mysql));
	res=mysql_store_result(&mysql);
	cPasswd[0]=0;
	if((field=mysql_fetch_row(res)))
		strcpy(cPasswd,field[0]);
	mysql_free_result(res);

	
	return(cPasswd);

}//char *cGetPasswd(char *cLogin)


void SSLCookieLogin(void)
{
	char *ptr,*ptr2;

	//Parse out login and passwd from cookies
#ifdef SSLONLY
	if(getenv("HTTPS")==NULL) 
		mysqlBind("Non SSL access denied");
#endif

	if(getenv("HTTP_COOKIE")!=NULL)
		strncpy(cCookie,getenv("HTTP_COOKIE"),1022);
	
	if(cCookie[0])
	{

	if((ptr=strstr(cCookie,"mysqlBindLogin=")))
	{
		ptr+=strlen("mysqlBindLogin=");
		if((ptr2=strchr(ptr,';')))
		{
			*ptr2=0;
			strncpy(cLogin,ptr,99);
			*ptr2=';';
		}
		else
		{
			strncpy(cLogin,ptr,99);
		}
	}
	if((ptr=strstr(cCookie,"mysqlBindPasswd=")))
	{
		ptr+=strlen("mysqlBindPasswd=");
		if((ptr2=strchr(ptr,';')))
		{
			*ptr2=0;
			strncpy(cPasswd,ptr,20);
			*ptr2=';';
		}
		else
		{
			strncpy(cPasswd,ptr,20);
		}
	}
	
	}//if cCookie[0] time saver

	if(!ValidLogin(1))
		htmlSSLLogin();

	strncpy(cUser,cLogin,41);
	GetPLAndClient(cUser);
	if(!uPermLevel || !uLoginClient)
		mysqlBind("Access denied");
	cPasswd[0]=0;
	uSSLCookieLogin=1;

}//SSLCookieLogin()


void htmlSSLLogin(void)
{
        Header_ism3("",0);

	printf("<p>\n");
	printf("Login: <input type=text size=20 maxlength=98 name=cLogin>\n");
	printf(" Passwd: <input type=password size=20 maxlength=20 name=cPasswd>\n");
	printf("<input type=submit name=function value=Login>\n");

        Footer_ism3();

}//void htmlSSLLogin(void)



void EncryptPasswdWithSalt(char *pw, char *salt)
{
	char passwd[102]={""};
	char *cpw;
			
	strcpy(passwd,pw);
				
	cpw=crypt(passwd,salt);

	strcpy(pw,cpw);

}//void EncryptPasswdWithSalt(char *pw, char *salt)


void GetPLAndClient(char *cUser)
{
        MYSQL_RES *res;
        MYSQL_ROW field;

	sprintf(query,"SELECT uPerm,uCertClient FROM tAuthorize WHERE cLabel='%s'",cUser);
	mysql_query(&mysql,query);
	if(mysql_errno(&mysql))
		mysqlBind(mysql_error(&mysql));
	res=mysql_store_result(&mysql);
	if(mysql_num_rows(res))
	{
		field=mysql_fetch_row(res);
		sscanf(field[0],"%d",&uPermLevel);
		sscanf(field[1],"%u",&uLoginClient);
	}
	mysql_free_result(res);

}//void GetPLAndClient()


void GetClientOwner(unsigned uClient, unsigned *uOwner)
{
        MYSQL_RES *res;
        MYSQL_ROW field;

        char query[254];

        sprintf(query,"SELECT uOwner FROM tClient WHERE uClient=%u",uClient);

        mysql_query(&mysql,query);
        if(mysql_errno(&mysql))
                mysqlBind(mysql_error(&mysql));
        res=mysql_store_result(&mysql);
        *uOwner=0;
        if((field=mysql_fetch_row(res)))
                sscanf(field[0],"%u",uOwner);
        mysql_free_result(res);

}//void GetClientOwner(unsigned uClient, unsigned *uOwner)


#ifdef USECONF

void ReadAndSetConfFileGlobalVars(void)
{
	FILE *fp;
	char cBuffer[256];
	char *cp;

	if((fp=fopen(USECONF,"r")))
	{
		while(fgets(cBuffer,255,fp)!=NULL)
		{
			if(cBuffer[0]=='#') continue;

			//Cut off \n
			cBuffer[strlen(cBuffer)-1]=0;
/*
char gcDbIp[16]={""};
char *gcEffectiveDbIp=DBIP;
char gcDbName[100]={DBNAME};
char gcDbLogin[100]={DBLOGIN};
char gcDbPasswd[32]={DBPASSWD};
unsigned guDbPort=0;
char gcDbSocket[100]={""};
char *gcEffectiveDbSocket=DBSOCKET;
*/


			if((cp=strstr(cBuffer,"gcDbIp=")))
			{
				sprintf(gcDbIp,"%.15s",cp+7);
				gcEffectiveDbIp=gcDbIp;
			}
			else if((cp=strstr(cBuffer,"gcDbName=")))
			{
				sprintf(gcDbName,"%.99s",cp+9);
			}
			else if((cp=strstr(cBuffer,"gcDbLogin=")))
			{
				sprintf(gcDbLogin,"%.99s",cp+10);
			}
			else if((cp=strstr(cBuffer,"gcDbPasswd=")))
			{
				sprintf(gcDbPasswd,"%.99s",cp+11);
			}
			else if((cp=strstr(cBuffer,"gcDbPort=")))
			{
				sscanf(cp+9,"%u",&guDbPort);
			}
			else if((cp=strstr(cBuffer,"gcDbSocket=")))
			{
				sprintf(gcDbSocket,"%.15s",cp+11);
				gcEffectiveDbSocket=gcDbSocket;
			}
		}
	}

	//Debug only
	//sprintf(query,"gcEffectiveDbIp=%s gcDbName=%s gcDbLogin=%s gcDbPasswd=%s gcEffectiveDbSocket=%s guDbPort=%u",gcEffectiveDbIp,gcDbName,gcDbLogin,gcDbPasswd,gcEffectiveDbSocket,guDbPort);
	//mysqlBind(query);

}//void ReadAndSetConfFileGlobalVars(void)

#endif
