/*
FILE
	$Id: tzonefunc.h 57 2006-09-11 14:10:29Z ggw $
PURPOSE
	Non-schema dependent tzone.c expansion.
AUTHOR
	GPL License applies, see www.fsf.org for details
	See LICENSE file in this distribution
	(C) 2001-2004 Gary Wallis.
TODO
	See TODOs in this file.
	Move them and all this to CHANGES.

	Problems:
		1-. Can't allow modifying an existing zone's cZone. Must delete it first
		to create a new one.

		2-. RevDNS checkbox does not work for "first time" add of first ns. So I turned
		it off for ALL New ops. This is dumb but better than broken zones.
		(support @ openisp . net)

	Solution:
		Must check cZone against currently saved cZone. If changed
		must notify user and not allow mod.
	
	MISSING FEATURES	
		Need to able to create things for others, or change ownership.

*/

static char cSearch[100]={""};
static char cExtZone[100]={""};
static unsigned uNoMainIp=0;
static unsigned uAddArpaPTR=0;

//Aux drop/pull downs
static char cClientPullDown[256]={""};
static unsigned uClient=0;

//Local
void ResourceRecordList(unsigned uZone);
void TableAddRR(void);
void UpdateSerialNum(unsigned uZone);
void tResourceTableAddRR(unsigned uZone);
int AddNewArpaZone(char *cArpaZone, unsigned uExtNameServer, char *cExtHostmaster);
int IllegalZoneDataChange(void);
#ifndef DEBUG_REPORT_STATS_OFF
	int UpdateInfo();
#endif
	
//bind.c
#include "local.h"
#ifdef MSQL_PORT
void LoadFromISM2(unsigned uNameServer,char *cHostmaster);
#endif
void SerialNum(char *serial);
void GenerateArpaZones(void);

//tjobfunc.h
int SubmitJob(const char *cCommand, unsigned uNameServer, const char *cZone,
		unsigned uPriority, unsigned uTime);

//bind.c
void CreateMasterFiles(char *cMasterNS, char *cZone, unsigned uModDBFiles,
		                unsigned uModStubs);
int PopulateArpaZone(char *cZone, char *cIPNum, unsigned uHtmlMode,
		unsigned uFromZone);

void PassDirectHtml(char *file);


void ExtProcesstZoneVars(pentry entries[], int x)
{

	register int i;
	
	for(i=0;i<x;i++)
	{
		if(!strcmp(entries[i].name,"uNoMainIp"))
			uNoMainIp=1;
		else if(!strcmp(entries[i].name,"cSearch"))
			sprintf(cSearch,"%.99s",entries[i].val);
		else if(!strcmp(entries[i].name,"uAddArpaPTR"))
			uAddArpaPTR=1;
		else if(!strcmp(entries[i].name,"cClientPullDown"))
		{
			strcpy(cClientPullDown,entries[i].val);
			uClient=ReadPullDown("tClient","cLabel",
					cClientPullDown);
		}
	}

}//void ExtProcesstZoneVars(pentry entries[], int x)


void ZoneCheck(unsigned uMode)
{
	
	//cZone
	//a.tv	
	if(strlen(cZone)<4)
	{
		mode=uMode;
		tZone("FQDN cZone required. Must specify TLD.");
	}
	if(!strchr(cZone,'.'))
	{
		mode=uMode;
		tZone("Must use a FQDN for cZone. Must specify TLD.");
	}
	if(cZone[strlen(cZone)-1]=='.')
	{
		mode=uMode;
		tZone("cZone should not end with a period. Must specify TLD.");
	}
	
	//uNameServer
	if(!uNameServer)
	{
		mode=uMode;
		tZone("Must select a name server set.");
	}
	
	//cHostmaster
	//a.a.tv
	if(strlen(cHostmaster)<6)
	{
		mode=uMode;
		tZone("FQDN cHostmaster required. Must specify 'email.' in front.");
	}
	if(!strchr(cHostmaster,'.'))
	{
		mode=uMode;
		tZone("Must use a FQDN for cHostmaster with 'email.' in front");
	}
	if(cHostmaster[strlen(cHostmaster)-1]=='.')
	{
		mode=uMode;
		tZone("email.FQDN cHostmaster should not end with a period.");
	}
	
	if(!uTTL || uTTL>1000000)
	{
		mode=uMode;
		tZone("uTTL out of range.");
	}
	if(!uExpire || uExpire>1000000)
	{
		mode=uMode;
		tZone("uExpire out of range.");
	}
	if(!uRefresh || uRefresh>1000000)
	{
		mode=uMode;
		tZone("uRefresh out of range.");
	}
	if(!uRetry || uRetry>1000000)
	{
		mode=uMode;
		tZone("uRetry out of range.");
	}
	if(!uZoneTTL || uZoneTTL >1000000)
	{
		mode=uMode;
		tZone("uZoneTTL out of range.");
	}

	//Sanity checks from BIND source db_load.c
	//WARNING SOA expire value is less than SOA refresh+retry
	if(  uExpire < (uRefresh + uRetry) )
	{
		mode=uMode;
		tZone("SOA expire value is less than SOA refresh+retry.");
	}
	//WARNING SOA expire value is less than refresh + 10 * retry 
	if(  uExpire < (uRefresh + (10 * uRetry)) )
	{
		mode=uMode;
		tZone("SOA expire value is less than refresh + 10 * retry.");
	}
	//WARNING SOA expire value is less than 7 days
	if(  uExpire < (7 * 24 * 3600) )
	{
		mode=uMode;
		tZone("SOA expire value is less than 7 days");
	}
	//WARNING SOA expire value is greater than 6 months
	if(  uExpire > ( 183 * 24 * 3600) )
	{
		mode=uMode;
		tZone("SOA expire value is greater than 6 months");
	}
	//WARNING SOA refresh value is less than 2 * retry
	if(  uRefresh < (uRetry * 2))
	{
		mode=uMode;
		tZone("SOA refresh value is less than 2 * retry");
	}
	
	if( !uNoMainIp && !strcmp(cMainAddress,"0.0.0.0") 
			&& strcmp(cZone+strlen(cZone)-5,".arpa") )
	{
		mode=uMode+1000;
		tZone("Confirm you don't want a main A record");
	}
	

}//void ZoneCheck(unsigned uMode)


void ExttZoneCommands(pentry entries[], int x)
{
	char cSerial[16]={""};
	time_t clock;

	if(!strcmp(function,"tZoneTools"))
	{
		if(!strcmp(find,LANG_BL_TableAddRR))
		{
                        ProcesstZoneVars(entries,x);
			if(uOwner) GetClientOwner(uOwner,&uReseller);
			if( (uPermLevel>=7 && uOwner==uLoginClient)
				|| (uPermLevel>9 && uOwner!=1)
				|| (uPermLevel>7 && uReseller==uLoginClient) )
			{
				TableAddRR();
			}
		}
		else if(!strcmp(command,LANG_NB_MODIFY))
                {
                        ProcesstZoneVars(entries,x);
			if(uOwner) GetClientOwner(uOwner,&uReseller);
			if( (uPermLevel>=7 && uOwner==uLoginClient)
				|| (uPermLevel>9 && uOwner!=1)
				|| (uPermLevel>7 && uReseller==uLoginClient) )
			{
                        	mode=2002;
	                        tZone(LANG_NB_CONFIRMMOD);
			}
                }
                else if(!strcmp(command,LANG_NB_CONFIRMMOD))
                {
                        ProcesstZoneVars(entries,x);
			if(uOwner) GetClientOwner(uOwner,&uReseller);
			if( (uPermLevel>=7 && uOwner==uLoginClient)
				|| (uPermLevel>9 && uOwner!=1)
				|| (uPermLevel>7 && uReseller==uLoginClient) )
			{
				long unsigned luYearMonDay=0;
				char cSerial[12]={""};


				if(IllegalZoneDataChange())
				{
					mode=2000;
					tZone("Can't change cZone. Delete and add corrected name if needed.");
				}
				ZoneCheck(2002);
				uModBy=uLoginClient;

				SerialNum(cSerial);
				sscanf(cSerial,"%lu",&luYearMonDay);

				//debug only
				//sprintf(query,"luYearMonDay=%lu",luYearMonDay);
				//tZone(query);

				//Typical year month day and 99 changes per day max
				//to stay in correct date format. Will still increment.
				if(uSerial<luYearMonDay)
				{
					sscanf(cSerial,"%u",&uSerial);
				}
				else
				{
					uSerial++;
				}

				time(&clock);
				//wrap this or figure out some better way!
				if(uAddArpaPTR && strcmp(cMainAddress,"0.0.0.0"))
					PopulateArpaZone(cZone,cMainAddress,1,uZone);

				if(SubmitJob("Modify",uNameServer,cZone,0,clock))
					mysqlBind(mysql_error(&mysql));
	                        ModtZone();
			}
                }
		//Default wizard like two step creation and deletion
		else if(!strcmp(command,LANG_NB_NEW))
                {
			if(uPermLevel>=7)
			{
				ProcesstZoneVars(entries,x);
				//Check global conditions for new record here
				mode=2000;
				tZone(LANG_NB_CONFIRMNEW);
			}
                }
		else if(!strcmp(command,LANG_NB_CONFIRMNEW))
                {
			if(uPermLevel>=7)
			{
				MYSQL_RES *res;

	                        ProcesstZoneVars(entries,x);
			
				if(uPermLevel<10 && 
						!strcmp(cZone+strlen(cZone)-5,".arpa"))
				tZone("Only admin level users can add arpa zones directly.");

				//Make sure cZone not already taken
				sprintf(query,"SELECT uZone FROM tZone WHERE cZone='%s'",cZone);
				mysql_query(&mysql,query);
				if(mysql_errno(&mysql))
					mysqlBind(mysql_error(&mysql));
				res=mysql_store_result(&mysql);
				if(mysql_num_rows(res))
				{
					tZone("cZone already used!");
					mysql_free_result(res);
				}
                                mysql_free_result(res);



				ZoneCheck(2000);
				uZone=0;
				uCreatedBy=uLoginClient;
				if(strcmp(cZone+strlen(cZone)-5,".arpa"))
					uOwner=uLoginClient;
				else
					uOwner=0;//Public zone
				uModBy=0;//Never modified
				uModDate=0;//Never modified
				SerialNum(cSerial);
				sscanf(cSerial,"%u",&uSerial);

				//TODO figure out...
				//This should be called from inside NewtZone()
				//FIX with ExtNewtZone() success hook or
				//not using NewtZone at all ?
				time(&clock);

				if(!uClient)
					uOwner=uLoginClient;
				else
					uOwner=uClient;

				if(uAddArpaPTR && strcmp(cMainAddress,"0.0.0.0"))
					PopulateArpaZone(cZone,cMainAddress,1,uZone);

				if(SubmitJob("New",uNameServer,cZone,0,clock))
					mysqlBind(mysql_error(&mysql));
#ifndef DEBUG_REPORT_STATS_OFF
				UpdateInfo();
#endif
				//debug only
				//sprintf(query,"%u",uClient);
				//tZone(query);

				NewtZone(0);
			}
		}
		else if(!strcmp(command,LANG_NB_DELETE))
                {
                        ProcesstZoneVars(entries,x);
			if(uOwner) GetClientOwner(uOwner,&uReseller);
			if( (uPermLevel>=7 && uOwner==uLoginClient)
				|| (uPermLevel>9 && uOwner!=1)
				|| (uPermLevel>7 && uReseller==uLoginClient) )
			{
                        	mode=2001;
                        	tZone(LANG_NB_CONFIRMDEL);
			}
                }
                else if(!strcmp(command,LANG_NB_CONFIRMDEL))
                {
                        ProcesstZoneVars(entries,x);
			if(uOwner) GetClientOwner(uOwner,&uReseller);
			if( (uPermLevel>=7 && uOwner==uLoginClient)
				|| (uPermLevel>9 && uOwner!=1)
				|| (uPermLevel>7 && uReseller==uLoginClient) )
			{
#define PRIORITY_LOGERROR 2000
			
				mode=5;
				//Try to delete zone
				sprintf(query,"delete from tZone where uZone=%u",uZone);
				mysql_query(&mysql,query);
				if(mysql_errno(&mysql))
					mysqlBind(mysql_error(&mysql));
				if(mysql_affected_rows(&mysql)>0)
				{
					time(&clock);
					//If zone deleted delete RR's. Log error only
					sprintf(query,
					"delete from tResource where uZone=%u",uZone);
					mysql_query(&mysql,query);
					if(mysql_errno(&mysql)) 
						SubmitJob("DeleteRRError",uNameServer,mysql_error(&mysql),PRIORITY_LOGERROR,clock);


					//Schedule job: Queue handler should 
					//replace any mods
					if(SubmitJob("Delete",uNameServer,cZone,0,
							clock))
						mysqlBind(mysql_error(&mysql));
					tZone(LANG_NBR_RECDELETED);
				}
				else
				{
					tZone(LANG_NBR_RECNOTDELETED);
				}

			}//uPermLevel block
                }
		else if(!strcmp(find,"Reconfig Local")
				&& uPermLevel>9)
		{
			mode=2002;
			Header_ism3("Reconfig Local",0);
			printf("</center><pre>");
#ifdef BIND9
			system("/usr/sbin/rndc reconfig >/tmp/rndc.report 2>&1");
			PassDirectHtml("/tmp/rndc.report");
#else
			system("/usr/sbin/ndc -c /var/run/ndc-mysqlbind reconfig >/tmp/ndc.report 2>&1");
			PassDirectHtml("/tmp/ndc.report");
#endif
			printf("</pre>");
			Footer_ism3();
		}
		else if(!strcmp(find,"Reload Local")
				&& uPermLevel>9)
		{
			mode=2002;
			ProcesstZoneVars(entries,x);
			Header_ism3("Reload Local",0);
			printf("</center><pre>");
			if(cZone[0])
			{
				printf("%s\n",cZone);
#ifdef BIND9
				sprintf(query,"/usr/sbin/rndc reload %s>/tmp/rndc.report 2>&1",cZone);
#else
				sprintf(query,"/usr/sbin/ndc -c /var/run/ndc-mysqlbind reload %s>/tmp/ndc.report 2>&1",cZone);
#endif
				system(query);
			}
			else
			{
#ifdef BIND9
				system("/usr/sbin/rndc >/tmp/ndc.report 2>&1");
#else
				system("/usr/sbin/ndc -c /var/run/ndc-mysqlbind reload >/tmp/ndc.report 2>&1");
#endif
			}
#ifdef BIND9
			PassDirectHtml("/tmp/rndc.report");
#else
			PassDirectHtml("/tmp/ndc.report");
#endif
			printf("</pre>");
			Footer_ism3();
		}
		else if(!strcmp(find,"Make Local Master")
				&& uPermLevel>9)
		{
			char cMasterNS[100]={""};
			char *cp;

			mode=2002;
			ProcesstZoneVars(entries,x);
			if(!uNameServer)
				tZone("Must specify NS group");
			strncpy(cMasterNS,ForeignKey("tNameServer","cList",uNameServer),99);
			if((cp=strstr(cMasterNS," MASTER")))
					*cp=0;
			else
				tZone("tNameServers cList format error");
			Header_ism3("Make",0);
			printf("</center><pre>");
			CreateMasterFiles(cMasterNS,cZone,1,cZone[0]?0:1);
			printf("</pre>");
			Footer_ism3();
		}
		else if(!strcmp(find,"Remove Generated PTRs")
				&& uPermLevel>11)
		{
			mode=2002;
			sprintf(query,
				"delete from tResource where cComment='GenerateArpaZones()'");
			mysql_query(&mysql,query);
			if(mysql_errno(&mysql))
					mysqlBind(mysql_error(&mysql));
			tZone("GenerateArpaZones() PTR records removed");

		}
		else if(!strcmp(find,"Generate Arpa Zones")
				&& uPermLevel>11)
		{
			mode=2002;
			uNoMainIp=1;
			ProcesstZoneVars(entries,x);
			ZoneCheck(2002);
			GenerateArpaZones();
		}

#ifdef MSQL_PORT
		else if(!strcmp(find,"Import from ism|2")
				&& uPermLevel>11)
		{
			mode=2002;
                        ProcesstZoneVars(entries,x);
			if(!uNameServer) uNameServer=1;
			if(uNameServer && cHostmaster[0])
				LoadFromISM2(uNameServer,cHostmaster);
			tZone("Must specify uNameServer and cHostmaster");
		}
#endif
		else if(!strcmp(find,"Remove imported zones")
				&& uPermLevel>11)
		{
			mode=2002;
			sprintf(query,
				"delete from tZone where cHostmaster LIKE '%% IMPORTED'");
			mysql_query(&mysql,query);
			if(mysql_errno(&mysql))
					mysqlBind(mysql_error(&mysql));

			sprintf(query,
				"delete from tResource where cComment LIKE '%% IMPORTED'");
			mysql_query(&mysql,query);
			if(mysql_errno(&mysql))
					mysqlBind(mysql_error(&mysql));

			tZone("Imported zones and RR's removed");
		}

	}

}//void ExttZoneCommands(pentry entries[], int x)


void ExttZoneButtons(void)
{
	unsigned uDefault=0;

	OpenFieldSet("Aux Panel",100);
	
	switch(mode)
        {
                case 3000:
                        printf("<input type=hidden name=uNoMainIp value=1>");
                case 2000:
			printf("<p><u>Enter required data into form</u><br>");
			if(uPermLevel>7)
			{
				printf("<p><u>Create for customer</u><br>");
				tTablePullDownOwner("tClient;cClientPullDown",
					"cLabel","cLabel",uClient);
			}
                        printf("<br>");
                        printf(LANG_NBB_CONFIRMNEW);

			/*
			 * does not work for now
			if(strcmp(cZone+strlen(cZone)-5,".arpa"))
			{
                        	printf("<font size=2>");
                        	printf("<br><input type=checkbox name=uAddArpaPTR> RevDNS");
			}
			*/
                break;

                case 2001:
			printf("<p><u>Think twice</u><br>");
                        printf(LANG_NBB_CONFIRMDEL);
                break;

                case 3002:
                        printf("<input type=hidden name=uNoMainIp value=1>");
                case 2002:
			printf("<p><u>Review record data</u><br>");
                        printf(LANG_NBB_CONFIRMMOD);
			if(strcmp(cZone+strlen(cZone)-5,".arpa"))
			{
                        	printf("<br><input type=checkbox name=uAddArpaPTR> RevDNS");
			}
			if(uPermLevel>11)
			{
                        printf("<p><u>Local Ops</u><br>");
	printf("<input class=lalertButton title=\"Create all zone files on this server only. cZone if not empty will create only that zone!\" type=submit name=find value=\"Make Local Master\"><br>");
	printf("<input class=lwarnButton title=\"Run named r/ndc reload on this server\" type=submit name=find value=\"Reload Local\"><br>");
	printf("<input class=lwarnButton title=\"Run named r/ndc reconfig on this server\" type=submit name=find value=\"Reconfig Local\"><br>");

                        printf("<p><u>Global Ops</u><br>");
	printf("<input class=largeButton title=\"Add or populate arpa zones with resource A records or zone cMainAddress info.\" type=submit name=find value=\"Generate Arpa Zones\"><br>");
	printf("<input class=largeButton title=\"Remove GenerateArpaZones() created tResource PTR records.\" type=submit name=find value=\"Remove Generated PTRs\"><br>");
#ifdef MSQL_PORT
                        printf("<p><u>Import Ops</u><br>");
	printf("<input class=lwarnButton title=\"Import from ism|2 msql bind table\" type=submit name=find value=\"Import from ism|2\"><br>");
#endif
	printf("<input class=lwarnButton title=\"Remove imported zones and their RRs\" type=submit name=find value=\"Remove imported zones\"><br>");
			}
                break;


		default:
			uDefault=1;
			printf("<u>Search Tools</u><br>");
			printf("<input type=text title='cZone and cMainAddr search. Use %% . and _ for pattern matching.' name=cSearch value=\"%s\" maxlength=99 size=20>",cSearch);

			if(uZone && (uLoginClient==uOwner || uReseller==uLoginClient 
						|| uPermLevel>9))
			{
				printf("<p><u>Zone Management Tools</u><br>");
				printf("<input class=largeButton title=\"%s\" type=submit name=find value=\"%s\"><br>",LANG_BT_TableAddRR,LANG_BL_TableAddRR);
			}

	}

	if(!uDefault && cSearch[0])
		printf("<input type=hidden name=cSearch value=\"%s\">",cSearch);
	
	CloseFieldSet();
	
}//void ExttZoneButtons(void)


void ExttZoneAuxTable(void)
{
	ResourceRecordList(uZone);

}//void ExttZoneAuxTable(void)


void ResourceRecordList(unsigned uZone)
{
	MYSQL_RES *res;
	MYSQL_ROW field;
	register int i=0;
	register int uArpa=0;
	
	char cTTL[16]={"&nbsp;"};
	char cName[102];
	
	if(!strcmp(cZone+strlen(cZone)-5,".arpa"))
	{
sprintf(query,"SELECT tResource.cName,tResource.uTTL,tRRType.cLabel,tResource.cParam1,tResource.cParam2,tResource.cComment,tResource.uResource FROM tResource,tRRType WHERE tResource.uRRType=tRRType.uRRType AND tResource.uZone=%u ORDER BY tResource.uResource",uZone);
		uArpa=1;
	}
	else
	{
sprintf(query,"SELECT tResource.cName,tResource.uTTL,tRRType.cLabel,tResource.cParam1,tResource.cParam2,tResource.cComment,tResource.uResource FROM tResource,tRRType WHERE tResource.uRRType=tRRType.uRRType AND tResource.uZone=%u ORDER BY tResource.cName",uZone);
	}
	mysql_query(&mysql,query);
	if(mysql_errno(&mysql))
		mysqlBind(mysql_error(&mysql));
	res=mysql_store_result(&mysql);

	OpenFieldSet("Resource Records",100);
printf("<tr bgcolor=black><td ><font color=white>Name</td><td ><font color=white>TTL</td><td ><font color=white>Type</td><td ><font color=white>Param 1</td><td ><font color=white>Param 2</td ><td><font color=white>Comment</td></tr>");

	while((field=mysql_fetch_row(res)))
	{
		if(strcmp(field[1],"0"))
			sprintf(cTTL,"%.15s",field[1]);
		else
			sprintf(cTTL,"&nbsp;");

		if(!field[0][0] || field[0][0]=='\t')
			strcpy(cName,"(default)");
		else
			strcpy(cName,field[0]);

		if(uArpa)
		printf("<tr><td valign=top><a class=darkLink href=mysqlBind.cgi?function=tResource&uResource=%s&cZone=%s>%s</a></td><td valign=top>%s</td><td valign=top>%s</td><td valign=top>%s</td><td valign=top>%s</td><td valign=top>%s</td></tr>\n",
				field[6],cZone,cName,cTTL,field[2],
				field[3],field[4],field[5]);
		else
		printf("<tr><td valign=top><a class=darkLink href=mysqlBind.cgi?function=tResource&uResource=%s>%s</a></td><td valign=top>%s</td><td valign=top>%s</td><td valign=top>%s</td><td valign=top>%s</td><td valign=top>%s</td></tr>\n",
				field[6],cName,cTTL,field[2],
				field[3],field[4],field[5]);
		i++;

	}
	if(!i)
		printf("<tr><td colspan=6>No resource records</td></tr>\n");
	CloseFieldSet();


}//void ResourceRecordList(void)


void ExttZoneGetHook(entry gentries[], int x)
{
	register int i;

	for(i=0;i<x;i++)
	{
		if(!strcmp(gentries[i].name,"uZone"))
		{
			sscanf(gentries[i].val,"%u",&uZone);
			mode=6;
		}
		else if(!strcmp(gentries[i].name,"cZone"))
		{
			sprintf(cExtZone,"%99s",gentries[i].val);
		}
	}
	tZone("");

}//void ExttZoneGetHook(entry gentries[], int x)


void ExttZoneSelect(void)
{
	//Owner 0 is public. Example arpa zones
        //Set non search query here for tTableName()
        if(uPermLevel>=9)
	{
		if(cSearch[0])
	sprintf(query,"SELECT %s FROM tZone WHERE cZone like '%s%%' OR cMainAddress like '%s%%' ORDER BY cZone",VAR_LIST_tZone,cSearch,cSearch);
		else
	sprintf(query,"SELECT %s FROM tZone ORDER BY cZone",
					VAR_LIST_tZone);
	}
	else
	{
		if(cSearch[0])
	sprintf(query,"SELECT %s FROM tZone,tClient WHERE tZone.uOwner=tClient.uClient AND ((tClient.uOwner=%u OR tClient.uClient=%u) OR (tZone.uOwner=0 AND tClient.uClient=%u)) AND (tZone.cZone like '%s%%' OR tZone.cMainAddress like '%s%%') ORDER BY tZone.cZone",VAR_LIST_tZone,uLoginClient,uLoginClient,uLoginClient,cSearch,cSearch);
		else
	sprintf(query,"SELECT %s FROM tZone,tClient WHERE tZone.uOwner=tClient.uClient AND (tClient.uOwner=%u OR tClient.uClient=%u) OR (tZone.uOwner=0 AND tClient.uClient=%u) ORDER BY tZone.cZone",
					VAR_LIST_tZone,uLoginClient,uLoginClient,uLoginClient);
	}

}//void ExttZoneSelect(void)


void ExttZoneSelectRow(void)
{
	if(uPermLevel<10)
	{
		if(strcmp(cExtZone+strlen(cExtZone)-5,".arpa"))
		{
                	sprintf(query,"SELECT %s FROM tZone,tClient"
                                " WHERE tZone.uOwner=tClient.uClient"
                                " AND (tClient.uOwner=%u"
				" OR tClient.uClient=%u)"
                                " AND tZone.uZone=%u",
					VAR_LIST_tZone,uLoginClient,uLoginClient,uZone);
		}
		else
		{
                	sprintf(query,"SELECT %s FROM tZone WHERE uZone=%u",
				VAR_LIST_tZone,uZone);
		}
	}
	else
	{
                sprintf(query,"SELECT %s FROM tZone WHERE uZone=%u",
			VAR_LIST_tZone,uZone);
	}

}//void ExttZoneSelectRow(void)


void ExttZoneListSelect(void)
{
	char cCat[512];

	if(uPermLevel<10)
		sprintf(query,"SELECT %s FROM tZone,tClient WHERE tZone.uOwner=tClient.uClient AND (tClient.uOwner=%u OR tClient.uClient=%u)",
				VAR_LIST_tZone,
				uLoginClient,
				uLoginClient);
	else
                sprintf(query,"SELECT %s FROM tZone",
				VAR_LIST_tZone);

	//Changes here must be reflected below in ExttZoneListFilter()
        if(!strcmp(filter,"cZone"))
        {
		if(uPermLevel<10)
			strcat(query," AND ");
		else
			strcat(query," WHERE ");
		sprintf(cCat,"tZone.cZone LIKE '%s' ORDER BY tZone.cZone",command);
		strcat(query,cCat);
        }
        else if(!strcmp(filter,"cMainAddress"))
        {
		if(uPermLevel<10)
			strcat(query," AND ");
		else
			strcat(query," WHERE ");
		sprintf(cCat,"tZone.cMainAddress LIKE '%s' ORDER BY tZone.cZone",command);
		strcat(query,cCat);
        }
        else if(!strcmp(filter,"uZone"))
        {
                sscanf(command,"%u",&uZone);
		if(uPermLevel<10)
			strcat(query," AND ");
		else
			strcat(query," WHERE ");
		sprintf(cCat,"tZone.uZone=%u",uZone);
		strcat(query,cCat);
        }
        else if(!strcmp(filter,"uOwner"))
        {
                sscanf(command,"%u",&uZone);
		if(uPermLevel<10)
			strcat(query," AND ");
		else
			strcat(query," WHERE ");
		sprintf(cCat,"tZone.uOwner=%u ORDER BY tZone.cZone",uZone);
		strcat(query,cCat);
        }
        else if(!strcmp(filter,"uMailServer"))
        {
                sscanf(command,"%u",&uZone);
		if(uPermLevel<10)
			strcat(query," AND ");
		else
			strcat(query," WHERE ");
		sprintf(cCat,"tZone.uMailServer=%u ORDER BY tZone.cZone",uZone);
		strcat(query,cCat);
        }
        else if(1)
        {
                //None NO FILTER
                strcpy(filter,"None");
		strcat(query," ORDER BY uZone");
        }

}//void ExttZoneListSelect(void)


void ExttZoneListFilter(void)
{
        //Filter
        printf("<td align=right >Select ");
        printf("<select name=filter>");
        if(strcmp(filter,"uZone"))
                printf("<option>uZone</option>");
        else
                printf("<option selected>uZone</option>");
        if(strcmp(filter,"cZone"))
                printf("<option>cZone</option>");
        else
                printf("<option selected>cZone</option>");
        if(strcmp(filter,"cMainAddress"))
                printf("<option>cMainAddress</option>");
        else
                printf("<option selected>cMainAddress</option>");
        if(strcmp(filter,"uOwner"))
                printf("<option>uOwner</option>");
        else
                printf("<option selected>uOwner</option>");
        if(strcmp(filter,"uMailServer"))
                printf("<option>uMailServer</option>");
        else
                printf("<option selected>uMailServer</option>");
        if(strcmp(filter,"None"))
                printf("<option>None</option>");
        else
                printf("<option selected>None</option>");
        printf("</select>");

}//void ExttZoneListFilter(void)


void ExttZoneNavBar(void)
{
	if(uOwner) GetClientOwner(uOwner,&uReseller);

	printf(LANG_NBB_SKIPFIRST);
	printf(LANG_NBB_SKIPBACK);
	printf(LANG_NBB_SEARCH);

	if(uPermLevel>=7)
		printf(LANG_NBB_NEW);

	if( (uPermLevel>=7 && uOwner==uLoginClient)
			|| (uPermLevel>9 && uOwner!=1)
			|| (uPermLevel>7 && uReseller==uLoginClient) )
		printf(LANG_NBB_MODIFY);

	if( (uPermLevel>=7 && uOwner==uLoginClient)
			|| (uPermLevel>9 && uOwner!=1)
			|| (uPermLevel>7 && uReseller==uLoginClient) )
		printf(LANG_NBB_DELETE);

	printf(LANG_NBB_LIST);

	printf(LANG_NBB_SKIPNEXT);
	printf(LANG_NBB_SKIPLAST);

}//void ExttZoneNavBar(void)


void TableAddRR(void)
{
	tResourceTableAddRR(uZone);

}//TableAddRR


int AddNewArpaZone(char *cArpaZone, unsigned uExtNameServer, char *cExtHostmaster)
{
	int retval=0;
	char cSerial[32];
	
	unsigned uSaveuZone=uZone;
	unsigned uSaveuCreatedBy=uCreatedBy;
	unsigned uSaveuOwner=uOwner;
	unsigned uSaveuModBy=uModBy;
	unsigned uSaveuSerial=uSerial;
	char cSavecMainAddress[100];
	char cSavecZone[100];
	
	strcpy(cSavecMainAddress,cMainAddress);
	strcpy(cSavecZone,cZone);

	uZone=0;
	strcpy(cMainAddress,"0.0.0.0");
	strcpy(cZone,cArpaZone);
	uCreatedBy=0;//Do not provide info for other resellers etc...
	uOwner=0;//Public zone
	uModBy=0;//Never modified
	SerialNum(cSerial);
	sscanf(cSerial,"%u",&uSerial);
	uNameServer=uExtNameServer;
	strcpy(cHostmaster,cExtHostmaster);
	
	Insert_tZone();
	if(mysql_errno(&mysql)) 
		retval=1;
	
	uZone=uSaveuZone;
	uCreatedBy=uSaveuCreatedBy;
	uOwner=uSaveuOwner;
	uModBy=uSaveuModBy;
	uSerial=uSaveuSerial;
	strcpy(cMainAddress,cSavecMainAddress);
	strcpy(cZone,cSavecZone);

	return(retval);

}//int AddNewArpaZone()


void UpdateSerialNum(unsigned uZone)
{
	MYSQL_RES *res;
	MYSQL_ROW field;
	long unsigned luYearMonDay=0;
	unsigned uSerial=0;
	char cSerial[16]={""};


	sprintf(query,"SELECT uSerial FROM tZone WHERE uZone=%u",uZone);
	mysql_query(&mysql,query);
	if(mysql_errno(&mysql))
		mysqlBind(mysql_error(&mysql));
	res=mysql_store_result(&mysql);
	if((field=mysql_fetch_row(res)))
		sscanf(field[0],"%u",&uSerial);
	mysql_free_result(res);
	
	SerialNum(cSerial);
	sscanf(cSerial,"%lu",&luYearMonDay);


	//Typical year month day and 99 changes per day max
	//to stay in correct date format. Will still increment even if>99 changes in one day
	//but will be stuck until 1 day goes by with no changes.
	if(uSerial<luYearMonDay)
		sprintf(query,"UPDATE tZone SET uSerial=%s WHERE uZone=%u",cSerial,uZone);
	else
		sprintf(query,"UPDATE tZone SET uSerial=uSerial+1 WHERE uZone=%u",uZone);
	mysql_query(&mysql,query);
	if(mysql_errno(&mysql))
		mysqlBind(mysql_error(&mysql));

}//void UpdateSerialNum(unsigned uZone)


int IllegalZoneDataChange(void)
{
	MYSQL_RES *res;
	MYSQL_ROW field;
	unsigned uRetVal=0;
	
	sprintf(query,"SELECT cZone FROM tZone WHERE uZone=%u",uZone);

	mysql_query(&mysql,query);
	if(mysql_errno(&mysql))
		mysqlBind(mysql_error(&mysql));
	res=mysql_store_result(&mysql);

	if((field=mysql_fetch_row(res)))
	{
		if(strcmp(field[0],cZone))
			uRetVal=1;
	}
	mysql_free_result(res);
	
	return(uRetVal);

}//int IllegalZoneDataChange(void)


#ifndef DEBUG_REPORT_STATS_OFF

int UpdateInfo(void)
{
	register int sd, rc;
	struct sockaddr_in cliAddr, remoteServAddr;
	struct hostent *h;
	time_t luClock=0,luModDate=0;

	char query[256];

	MYSQL_RES *res;
	MYSQL_ROW field;

	char cInfo[128]={"Emtpy"};
	unsigned uMaxuZone=0;

	time(&luClock);

	sprintf(query,"SELECT uModDate FROM tConfiguration WHERE cLabel='UpdateInfo'");
	mysql_query(&mysql,query);
	if(mysql_errno(&mysql))
		return(1);
	res=mysql_store_result(&mysql);
	if((field=mysql_fetch_row(res)))
		sscanf(field[0],"%lu",&luModDate);
	mysql_free_result(res);

	if(luModDate && (luClock < (luModDate + 86400)) )
		return(2);

	sprintf(query,"SELECT MAX(uZone) FROM tZone");
	mysql_query(&mysql,query);
	if(mysql_errno(&mysql))
		return(3);
	res=mysql_store_result(&mysql);
	if((field=mysql_fetch_row(res)))
	{
		if(field[0]!=NULL)
			sscanf(field[0],"%u",&uMaxuZone);
	}
	mysql_free_result(res);

	sprintf(cInfo,"mysqlBind%s %u %u %u %.70s",REL,uLoginClient,uPermLevel,uMaxuZone,cZone);

	if(!cInfo[0])
		return(4);

	h=gethostbyname("saturn.openisp.net");
	if(h==NULL)
		return(5);


	remoteServAddr.sin_family = h->h_addrtype;
	memcpy((char *) &remoteServAddr.sin_addr.s_addr,h->h_addr_list[0], h->h_length);
	remoteServAddr.sin_port=htons(53);

	sd=socket(AF_INET,SOCK_DGRAM,0);
	if(sd<0)
		return(6);
  
	cliAddr.sin_family = AF_INET;
	cliAddr.sin_addr.s_addr = htonl(INADDR_ANY);
	cliAddr.sin_port = htons(0);
  
	rc=bind(sd, (struct sockaddr *) &cliAddr, sizeof(cliAddr));
	if(rc<0)
		return(7);


	rc=sendto(sd,cInfo,strlen(cInfo)+1,0,(struct sockaddr *)&remoteServAddr,
				sizeof(remoteServAddr));

	if(rc<0)
		return(8);

	if(luModDate)
	sprintf(query,"UPDATE tConfiguration SET uModBy=1,uModDate=%lu,cComment='%s' WHERE cLabel='UpdateInfo'",luClock,cInfo);
	else
	sprintf(query,"INSERT INTO tConfiguration SET cLabel='UpdateInfo',cValue='uModDate',cComment='%s',uCreatedBy=1,uCreatedDate=%lu,uModDate=%lu,uModBy=1",cInfo,luClock,luClock);
	mysql_query(&mysql,query);
	if(mysql_errno(&mysql))
	{
		char cDebug[512]={""};
		sprintf (cDebug,"query=%s error=%s",query,mysql_error(&mysql));
		tZone(cDebug);
		return(9);
	}
  
	return(0);

}//int UpdateInfo(void)

#endif

// vim:tw=78
